using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace GB_GUI_Loader
{
    /// <summary>
    /// Diese Klasse startet die in der INI-Datei angegebene Software. Zuvor berprft sie ob bereits Instanzen dieser Software in Bearbeitung sind und beendet diese zuerst.
    /// </summary>
    public static class Ini
    {
        /// <summary>
        /// Externe Methode zum auslesen einer INI-Datei
        /// </summary>
        /// <param name="lpAppName">Kategorie</param>
        /// <param name="lpKeyName">Schlsselname</param>
        /// <param name="lpDefault">Defaultwert des Schlssels</param>
        /// <param name="lpReturnString">String fr Rckgabewert</param>
        /// <param name="nSize">Gre des Strings</param>
        /// <param name="lpFilename">Dateiname</param>
        /// <returns></returns>
        [DllImport("KERNEL32.DLL", EntryPoint = "GetPrivateProfileStringW",
            SetLastError = true,
            CharSet = CharSet.Unicode, ExactSpelling = true,
            CallingConvention = CallingConvention.StdCall)]
        public static extern int GetPrivateProfileString(
            string lpAppName,
            string lpKeyName,
            string lpDefault,
            string lpReturnString,
            int nSize,
            string lpFilename);

        /// <summary>
        /// Externe Methode zum Schreiben einer INI-Datei
        /// </summary>
        /// <param name="lpAppName">Kategorie</param>
        /// <param name="lpKeyName">Schlsselname</param>
        /// <param name="lpString">Wert des Schlssels</param>
        /// <param name="lpFilename">Dateiname</param>
        /// <returns></returns>
        [DllImport("KERNEL32.DLL", EntryPoint = "WritePrivateProfileStringW",
            SetLastError = true,
            CharSet = CharSet.Unicode, ExactSpelling = true,
            CallingConvention = CallingConvention.StdCall)]
        public static extern int WritePrivateProfileString(
            string lpAppName,
            string lpKeyName,
            string lpString,
            string lpFilename);

        /// <summary>
        /// Liest den Wert eines Schlssels aus einer INI-Datei aus
        /// </summary>
        /// <param name="iniFile">Name INI-Datei</param>
        /// <param name="category">Kategorie</param>
        /// <param name="key">Schlssel der gelesen werden soll</param>
        /// <param name="defaultValue">Default-Wert, falls kein passender Eintrag gefunden wird</param>
        /// <returns>Wert des Schlssls bzw. Default-Wert</returns>
        public static string GetIniFileString(string iniFile, string category, string key, string defaultValue)
        {
            string returnString = new string(' ', 1024);
            GetPrivateProfileString(category, key, defaultValue, returnString, 1024, iniFile);
            return returnString.Split('\0')[0];
        }

        /// <summary>
        /// Liest alle Schlssel einer Kategorie einer INI-Datei ein
        /// </summary>
        /// <param name="iniFile">Name INI-Datei</param>
        /// <param name="category">Kategorie</param>
        /// <returns>Liste mit Schlsselbezeichnungen</returns>
        public static List<string> GetKeys(string iniFile, string category)
        {
            string returnString = new string(' ', 32768);
            GetPrivateProfileString(category, null, null, returnString, 32768, iniFile);
            List<string> result = new List<string>(returnString.Split('\0'));
            result.RemoveRange(result.Count - 2, 2);
            return result;
        }

        /// <summary>
        /// Liest alle Kategorien aus
        /// </summary>
        /// <param name="iniFile">Name INI-Datei</param>
        /// <returns>Liste mit den Kategoriebezeichnungen</returns>
        public static List<string> GetCategories(string iniFile)
        {
            string returnString = new string(' ', 65536);
            GetPrivateProfileString(null, null, null, returnString, 65536, iniFile);
            List<string> result = new List<string>(returnString.Split('\0'));
            result.RemoveRange(result.Count - 2, 2);
            return result;
        }
    }
}
