using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Diagnostics;
///Einstiegsklasse des Programms mit Methoden zum Starten und Beenden von Prozessen
namespace GB_GUI_Loader
{
    /// <summary>
    /// Diese Klasse startet die in der INI-Datei angegebene Software. Zuvor überprüft sie ob bereits Instanzen dieser Software in Bearbeitung sind und beendet diese zuerst.
    /// </summary>
    static class Program
    {
        /// <summary>
        /// Name der INI-Datei, aus der gelesen werden soll
        /// </summary>
        private static readonly string IniFileName = Application.StartupPath +  "\\GuiLoaderSetup.ini";
        /// <summary>
        /// Liest den Namen des zu startenden Prozess aus der INI-Datei (Prozessname OHNE Dateiendung)
        /// </summary>
        private static string GuiProcessName = Ini.GetIniFileString(IniFileName, "LoaderSetup", "ProcessName", "GUI");
        /// <summary>
        /// Liest den Pfad des zu startenden Prozesses aus der INI-Datei
        /// </summary>
        private static string GuiPfadName = Ini.GetIniFileString(IniFileName, "LoaderSetup", "GUIFilePath", @"C:\Visualisierung\GUI\GB-GUI.exe");
        
        /// <summary>
        /// Der Haupteinstiegspunkt für die Anwendung.
        /// </summary>
        [STAThread]
        static void Main()
        {
            GuiLoader();
        }

        /// <summary>
        /// Überprüft, ob schon Instanzen der GUI laufen und beendet diese.
        /// Anschließend wird eine neue Instanz gestartet
        /// </summary>
        private static void GuiLoader()
        {
            //Abfragen ob überhaupt Process am laufen ist
            bool ProcessExistiert = CheckIfAProcessIsRunning(GuiProcessName);

            //Process beenden
            if (ProcessExistiert)
            {
                stopProcess(GuiProcessName);
            }
            //Starten der GUI
            startProcess(GuiPfadName);
        }





        #region ______________________________________ Subroutinen ____________________________________
        /// <summary>
        /// Abfrage ob ein Processüberaupt läuft
        /// </summary>
        /// <param name="processname">Name des Prozesses</param>
        /// <returns>TRUE: Wenn mindestens eine Instanz vorhanden ist</returns>
        private static bool CheckIfAProcessIsRunning(string processname)
        {
            return Process.GetProcessesByName(processname).Length > 0;
        }

        /// <summary>
        /// Startet einen Process
        /// </summary>
        /// <param name="filename">Pfadangabe des zu startenden Prozesses</param>
        private static void startProcess(string filename)
        {
            Process P = new Process();
            P.StartInfo.FileName = filename;
            // hier kann z.B. eine Textdatei mit übergeben werden
            // P.StartInfo.Arguments = "Test.txt";
            try
            {
                P.Start();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message, "Fehler Process Start", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Stoppt alle Proccesse mit dem übergebenden Namen
        /// </summary>
        /// <param name="processname">Name des Prozesses</param>
        private static void stopProcess(string processname)
        {
            Process[] pp = Process.GetProcessesByName(GuiProcessName);
            foreach (Process p in pp)
            {
                p.CloseMainWindow();// Normales ende
                p.Kill();// sofort beenden
            }
        }
        #endregion

    }
}
